%{

#include <stdio.h>
#include <stdlib.h>
#include <math.h>		/* 数学関係の関数を利用 */

#include "node.h"
#include "eval.h"

extern int yylex();
extern int yyparse();
extern FILE* yyin;

void yyerror(const char* s);

Node *variable['z'-'a'+1];	/* 変数 'a' 〜 'z' の値を保持する領域 */
Node *function['Z'-'A'+1];	/* 関数 'A' 〜 'Z' の値を保持する領域 */

%}

%union {
  int ival;
  float fval;
  void *nval;
}

%token<ival> T_INT T_NAME T_FUNC
%token<fval> T_FLOAT
%token T_PLUS T_MINUS T_MULTIPLY T_DIVIDE T_LEFT T_RIGHT
%token T_L_BRA T_R_BRA
%token T_EQ T_DEF
%token T_NEWLINE T_QUIT
%token T_SIN
%token T_DOLLAR1
%token T_IF T_COMMA
%left T_BIT_OR
%nonassoc T_LESSTHAN
%left T_PLUS T_MINUS
%left T_MULTIPLY T_DIVIDE
%right T_POWER

%type<ival> l_value
%type<nval> expression

%start calculation

%%

calculation: 
	   | calculation line
;

line: T_NEWLINE
    | T_QUIT T_NEWLINE { printf("bye!\n"); exit(0); }
	| expression T_NEWLINE {
			printf("\tResult: ");
			print_number(eval($1,function,NULL));
			printf("\n");
		} 
	| l_value T_EQ expression T_NEWLINE {
		variable[$1]=eval($3,function,NULL);
		printf("\tResult: %c=", 'a' + $1);
		print_number(variable[$1]);
		printf("\n");
	}
	| T_FUNC T_DEF expression T_NEWLINE {
	  function[$1]=$3;
	  printf("\tResult: %c define\n", 'A' + $1);
 	}
;

expression
	  : T_INT				{ $$ = make_int_node ( $1 ); }
	  |	T_FLOAT             { $$ = make_float_node( $1 ); }
	  | T_DOLLAR1			{ $$ = make_node(T_DOLLAR1,0,NULL,NULL); }
| expression T_BIT_OR expression	{ $$ = make_node(T_BIT_OR,2,$1,$3); }
	  | expression T_PLUS expression	{ $$ = make_node(T_PLUS,2,$1,$3); }
	  | expression T_MINUS expression	{ $$ = make_node(T_MINUS,2,$1,$3); }
	  | expression T_MULTIPLY expression	{ $$ = make_node(T_MULTIPLY,2,$1,$3); }
	  | expression T_DIVIDE expression	{ $$ = make_node(T_DIVIDE,2,$1,$3); }
	  | expression T_LESSTHAN expression	{ $$ = make_node(T_LESSTHAN,2,$1,$3); }
	  | T_LEFT expression T_RIGHT		{ $$ = $2; }
	  | expression T_POWER expression	{
		$$ = make_node(T_POWER,2,$1,$3);
	  }
	  | T_SIN T_LEFT expression T_RIGHT    {
			$$ = make_node(T_SIN,1,$3,NULL);
		}
	  | T_FUNC T_LEFT expression T_RIGHT    {
//			$$ = make_node(T_FUNC, $1, $3, NULL);
			$$ = make_node(T_FUNC, 2, make_int_node($1), $3 );
//			$$ = make_node(T_FUNC, 2, function[$1], $3 );
		}
	  | T_IF T_LEFT expression T_COMMA expression T_COMMA expression T_RIGHT {
			$$ = make_node3( T_IF, 3, $3, $5, $7 );
		}
	  | l_value                               	 { $$ = variable[$1]; }
	;

l_value: T_NAME { $$ = $1; }
/*
	| T_L_BRA expression T_R_BRA           { $$ = $2; }
*/
	;

%%

int main() {
	int i;

	for ( i = 'a'; i <='z'; i++ ) {
	  variable[i-'a'] = make_float_node (0.0);
	}

	yyin = stdin;

	do { 
		yyparse();
	} while(!feof(yyin));

	return 0;
}

void yyerror(const char* s) {
	fprintf(stderr, "Parse error: %s\n", s);
	exit(1);
}

int ipower( int b, int n ) {
  int v = 1;
  int i;

  for ( i = 0; i < n; i++ ) {
	v *= b;
  }

  return v;
}

