/*
 * eval.c
 *	式の構文木を計算する
 *		計算結果は、最後には必ず、T_INT か T_FLOAT の値
 */

#include <stdio.h>
#include <stdlib.h>
#include "calc.tab.h"
#include "node.h"
#include "eval.h"

Node *eval ( Node *tree, Node *function[], Node *dollarValue ) {

  if ( tree == NULL ) {
	/* (*) エラー処理をきちんと行う必要がある */
	fprintf ( stderr, "Tree is NULL\n" );
	exit ( -1 );
  } else {
	switch ( tree -> type ) {
	case T_INT:
	case T_FLOAT:
	  return tree;
	  break;
	case T_IF:
	  {
		Node *op1 = eval ( (Node *)tree -> op1, function, dollarValue );

		if ( op1 -> type == T_INT && get_int(op1) == 0 ) {
		  /* 不成立 */
		  return eval( (Node *)tree -> op3, function, dollarValue );
		} else {
		  return eval( (Node *)tree -> op2, function, dollarValue );
		  /* 成立 */
		}
	  }
	  break;
	case T_DOLLAR1:
	  return dollarValue;
	case T_FUNC:
	  {
		Node *op1 = eval ( (Node *)tree -> op1, function, dollarValue );
		Node *op2 = eval ( (Node *)tree -> op2, function, dollarValue );

	  return eval ( function[get_int(op1)],
					function,
					eval(op2, function, dollarValue ) );
	  }
	  break;
	case T_BIT_OR:
	  {
	  Node *op1 = eval ( (Node *)tree -> op1, function, dollarValue );
	  Node *op2 = eval ( (Node *)tree -> op2, function, dollarValue );

	  if ( op1 -> type == T_INT && op2 -> type == T_INT  ) {
		  return make_int_node ( get_int( op1 ) | get_int( op2 ) );
	  } else {
		/* エラーにする必要がある */
		fprintf ( stderr, "invalid operand for '|' operator\n" ); 
		exit(-1);
	  }
	  }
	  break;
	case T_PLUS:
	case T_MINUS:
	case T_MULTIPLY:
	case T_DIVIDE:
	case T_LESSTHAN:
	{
	  Node *op1 = eval ( (Node *)tree -> op1, function, dollarValue );
	  Node *op2 = eval ( (Node *)tree -> op2, function, dollarValue );

	  if ( op1 -> type == T_INT && op2 -> type == T_INT ) {
		switch (  tree -> type ) {
		case T_PLUS:
		  return make_int_node ( get_int(op1) + get_int(op2) );
		case T_MINUS:
		  return make_int_node ( get_int(op1) - get_int(op2) );
		case T_MULTIPLY:
		  return make_int_node ( get_int(op1) * get_int(op2) );
		case T_DIVIDE:
		  /* (*) 本当割り算をするまえに 0 かどうかを判定する必要がある */
		  return make_int_node ( get_int(op1) - get_int(op2) );
		case T_LESSTHAN:
		  return make_int_node ( get_int(op1) < get_int(op2) );
		}
	  } else {
		double vop1;
		double vop2;

		if ( op1 -> type == T_INT ) {
		  vop1 = (double)get_int(op1);
		} else {
		  vop1 = get_float(op1);
		}

		if ( op2 -> type == T_INT ) {
		  vop2 = (double)get_int(op2);
		} else {
		  vop2 = get_float(op2);
		}

		switch (  tree -> type ) {
		case T_PLUS:
		  return make_float_node ( vop1 + vop2 );
		case T_MINUS:
		  return make_float_node ( vop1 - vop2 );
		case T_MULTIPLY:
		  return make_float_node ( vop1 * vop2 );
		case T_DIVIDE:
		  /* (*) 本当割り算をするまえに 0 かどうかを判定する必要がある */
		  return make_float_node ( vop1 / vop2 );
		case T_LESSTHAN:
		  return make_int_node ( vop1 < vop2 );
		}

	  }
	}
	  break;
	}
  }
}

#ifdef	DEBUG

void main(void) {
  Node *exp = make_node ( T_PLUS, 2,
						  make_int_node ( 2 ),
						  make_node ( T_PLUS, 2,
									  make_int_node ( 3 ),
									  make_int_node ( 4 ) 
									  ));
  /* 2 + 3 + 4 */

  print_number ( eval ( exp, NULL, NULL ) );

}

#endif
