/*
 * node.h
 */

#ifndef __NODE_H__
#define	__NODE_H__

/*	
 *	一つの Node は、4 つ部分からなり
 *		1 つ目は、これが何の Node かを示す
 *		2 つ目は、そのノードが最大「二つ」の枝を持つので、その個数
 *			# 「式」なので、「最大二つ」
 *			1 か 2 のどちらか ( Node Type の中に含める立場もある )
 *		3 つ目は、一本目の枝
 *		4 つ目は、あれば二本目の枝
 *	+-------+-------+-------+-------+
 *	|		|		|		|		|
 *	|		|		|		|		|
 *	|		|		|		|		|
 *	+-------+-------+-------+-------+
 */

typedef	struct	node	{
	int	type;		/* Node Type */
	int length;		/* 引数の個数 */
	void	*op1;	/* 一つ目の引数 */
	void	*op2;	/* 二つ目の引数 */
	void	*op3;	/* 三つ目の引数 */
} Node, *NodeP;

/*
 *
 *					|
 *				expression:A
 *		      /		|	  \	+(1,2)
 *		     /		|	   \
 *		    /		|		\
 *		   /		|		 \
 *		  /			|		  \
 *		 /			|		   \
 *		|			|			|
 *	expression:B	|		expression:C
 *		|	int(1)	|			|	int(2)
 *		T_INT		|			T_INT
 *		|			|			|
 *		1       	+           2
 *
 *[A]
 *	+-------+-------+-------+-------+
 *	|		|		|		|		|
 *	|	+	|	2	|		|		|		1 + 2
 *	|		|		|	|	|	|	|
 *	+-------+-------+---+---+---+---+
 *						|		|
 *						|		+-----------------------+
 *						|								|
 *						|								|
 *		[B]				v					[C]			v
 *		+-------+-------+-------+-------+	+-------+-------+-------+-------+
 *		|		|		|		|		|	|		|		|		|		|
 *		|T_INT	|	1	|		|		|	|T_INT	|	1	|		|		|
 *		|		|		|	|	|		|	|		|		|		|		|
 *		+-------+-------+---+---+-------+	+-------+-------+---+---+-------+
 *							|									|
 *							v									v
 *						+-------+							+-------+
 *						|		|							|		|
 *						|	1	|							|	2	|
 *						|		|							|		|
 *						+-------+							+-------+
 *
 *		1+2
 *			+
 *		   / \
 *		  i   i
 *		  |	  |
 *		  1	  2
 *
 *		1+2*3
 *
 *			+
 *		   / \
 *		  i   *
 *		  |	 / \
 *		  1	 i  i 
 *			 |  |
 *		     2  3
 *
 */

/*
 *
 */

#define	get_value(Type,Node)	(*((Type *)((Node)->op1)))
#define	get_int(Node)			get_value(int,Node)
#define	get_float(Node)			get_value(double,Node)

/*
 *
 */

extern	Node	*make_node3( int type, int length, void *op1, void *op2, void *op3 );

extern	Node	*make_node( int type, int length, void *op1, void *op2 );
extern	void print_number ( Node *node );
extern	Node	*make_int_node( int value );
extern	Node	*make_float_node( double value );

/*
 *
 */

#endif

